cbuffer pass_constants : register(b0)
{
  // view * projection matrix, world excluded since world space position is
  // part of the instance data
  float4x4 vp;
  
  float4 cam_up;
  float4 cam_right;
  
  // scale for particle size
  float scale;
  
  // 0 = don't use the texture for computing color values, 1 = use texture for
  // color values
  int use_texture;
}

// pre-multiplied alpha texture to use for particle coloring
Texture2D particle_texture : register(t0);

SamplerState tex_sampler : register(s0);

struct VertexShaderInput
{
  float4 position : POSITION;
  float2 uv       : TEXCOORD0;
};

struct VertexShaderOutput
{
  float4 position : SV_POSITION;
  float4 color    : COLOR0;
  float2 uv       : TEXCOORD0;
};

VertexShaderOutput VS(VertexShaderInput input,
  float4 pos : TEXCOORD1,float3 color : TEXCOORD2)
{
  VertexShaderOutput output;
  
  output.position =
    pos + scale
    * (input.position.x * cam_right + input.position.y * cam_up);
  output.position = mul(output.position,vp);
  output.color = float4(color,1);
  output.uv = input.uv;

  return output;
}

float4 PS(VertexShaderOutput input) : SV_Target
{
  //float4 color = tex2D(TextureSampler,input.uv);
  //return float4(input.color.rgb * color.r,color.r);
  //return tex2D(TextureSampler,input.uv);
  
  return use_texture * particle_texture.Sample(tex_sampler,input.uv)
    + (1 - use_texture) * input.color;
}
