﻿#include "Camera.h"
#include "GraphicsVector.h"

Camera::Camera(float aspect_ratio,XMFLOAT4 lookat,float range,float yaw,
  float pitch)
{
  m_look_at = lookat;
  m_range   = range;
  m_yaw     = yaw;
  m_pitch   = pitch;

  XMStoreFloat4x4(&m_proj,
    XMMatrixPerspectiveFovLH(XM_PIDIV4,aspect_ratio,.1f,100));
      
  Recalc();
}

void Camera::SetLookAtX(float x)
{
  m_look_at.x = x;
      
  Recalc();
}

void Camera::SetLookAtY(float y)
{
  m_look_at.y = y;
      
  Recalc();
}

void Camera::SetLookAtZ(float z)
{
  m_look_at.z = z;
      
  Recalc();
}

void Camera::SetOrbitRange(float range)
{
  m_range = range;
      
  Recalc();
}

void Camera::SetOrbitYaw(float yaw)
{
  m_yaw = yaw;
      
  Recalc();
}

void Camera::SetOrbitPitch(float pitch)
{
  m_pitch = pitch;
      
  Recalc();
}

const XMFLOAT4X4& Camera::GetProjection() const
{
  return m_proj;
}

const XMFLOAT4X4& Camera::GetView() const
{
  return m_view;
}

const XMFLOAT4& Camera::GetRight() const
{
  return m_right;
}

const XMFLOAT4& Camera::GetUp() const
{
  return m_up;
}

void Camera::Recalc()
{
  XMMATRIX convert = XMMatrixTranslation(0,0,-m_range)
    * XMMatrixRotationRollPitchYaw(m_pitch,m_yaw,0);
  
  // compute the camera position
  XMVECTOR pos = XMVector4Transform(XMLoadFloat4(&m_look_at),convert);
  XMStoreFloat4(&m_pos,pos);
  
  // compute the direction
  XMFLOAT4 tmp = m_look_at - m_pos;
  XMVECTOR dir = XMVector4Normalize(XMLoadFloat4(&tmp));
  XMStoreFloat4(&m_dir,dir);
  
  // compute the up vector
  XMVECTOR up = XMVector4Transform(XMVectorSet(0,1,0,0),convert);
  XMStoreFloat4(&m_up,up);
  
  // compute the right vector
  XMStoreFloat4(&m_right,XMVector3Cross(up,dir));

  XMStoreFloat4x4(&m_view,XMMatrixLookAtLH(pos,XMLoadFloat4(&m_look_at),up));
}
