#ifndef GAMEMAIN_H
#define GAMEMAIN_H

#include <map>
#include <vector>
#include "ServerSocket.h"

#include "Game.h"
#include "Effects/VSPSEffect.h"
#include "Buffers/VertexBuffer_PositionTexture.h"
#include "Buffers/IndexBuffer16.h"
#include "Buffers/InstanceBuffer.h"
#include "Buffers/VertexBufferArray.h"
#include "Texture.h"
#include "TextureSampler.h"

#include "Camera.h"

#include "ParticleSystem/ParticleSystem.h"
#include "Messages/ParameterIDs.h"
#include "ParticleSystem/EmitterShapes/EmitterShapes.h"
#include "ParticleSystem/EmitterShapes/EmitterShapeBase.h"


class GameMain : public Game
{
  public:
    GameMain(WCHAR* title,HINSTANCE instance);
    ~GameMain();
    
  protected:
    /// <summary>
    /// Loads the content for the game
    /// </summary>
    /// <param name="content">
    /// content manager to use for loading resources
    /// </param>
    void LoadContent();
    
    /// <summary>
    /// Unloads the content for the game before the dealloction of the instance
    /// </summary>
    /// <param name="content">
    /// content manager to use for unloading resources
    /// </param>
    void UnloadContent();
    
    /// <summary>
    /// Updates the game state
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the update
    /// </param>
    void Update(UINT ms);
    
    /// <summary>
    /// Draws the current frame
    /// </summary>
    /// <param name="ms">
    /// number of milliseconds since the last frame
    /// </param>
    void Draw(UINT ms);
    
  private:
    // disabled
    GameMain(const GameMain& cpy);
    GameMain& operator=(const GameMain& cpy);
    
    /// <summary>
    /// Takes care of setting a parameter.
    /// <remarks>
    /// This function exists to try to avoid cluttering the Update function
    /// </remarks>
    /// </summary>
    /// <param name="id">
    /// ID of the parameter to update
    /// </param>
    /// <param name="val">
    /// value to update the parameter to
    /// </param>
    void SetParam(ParameterIDs id,float val);
    
    /// <summary>
    /// Takes care of setting a parameter.
    /// <remarks>
    /// This function exists to try to avoid cluttering the Update function
    /// </remarks>
    /// </summary>
    /// <param name="id">
    /// ID of the parameter to update
    /// </param>
    /// <param name="val">
    /// value to update the parameter to
    /// </param>
    void SetParam(ParameterIDs id,const std::string& val);
    
    /// <summary>
    /// Takes care of setting a parameter.
    /// <remarks>
    /// This function exists to try to avoid cluttering the Update function
    /// </remarks>
    /// </summary>
    /// <param name="val">
    /// value to update the parameter to
    /// </param>
    void SetParam(std::vector<ColorSetEntry>& val);
    
    /// <summary>
    /// Takes care of setting a parameter.
    /// <remarks>
    /// This function exists to try to avoid cluttering the Update function
    /// </remarks>
    /// </summary>
    /// <param name="val">
    /// value to update the parameter to
    /// </param>
    void SetParam(std::vector<ColorSeqEntry>& val);
    
    /// <summary>
    /// server socket for listening for clients
    /// </summary>
    ServerSocket* m_server;
    
    /// <summary>
    /// connection to the client (only 1 client at a time is supported at the
    /// moment)
    /// </summary>
    TCPSocket* m_client;
    
    /// <summary>
    /// particle system to edit
    /// </summary>
    ParticleSystem* m_particle;
    
    /// <summary>
    /// map of emitter shape ids to emitter instances.  Due to the way emitters
    /// work, there is no need to have more than one instance of a particular
    /// shape of emitter
    /// </summary>
    std::map<EmitterShapes,EmitterShapeBase*> m_emitters;
    
    /// <summary>
    /// camera for viewing rendering
    /// </summary>
    Camera* m_cam;
};

#endif /* GAMEMAIN_H */
