#include <stdint.h>
#include "Message.h"
#include "MsgCloseConnection.h"
#include "MsgSetSingle.h"
#include "MsgSetString.h"
#include "MsgColorSet.h"
#include "MsgColorSeq.h"

Message::Message()
{
}

Message::~Message()
{
}

Message* Message::Receive(TCPSocket& socket)
{
  // check the version number
  int version;
  if (socket.ReadINT32N(version) != sizeof(version))
  {
    return NULL;
  }
  if (version != 1)
  {
    return NULL;
  }
  
  // get the length
  int length;
  if (socket.ReadINT32N(length) != sizeof(length))
  {
    return NULL;
  }
  if (length < 0)
  {
    return NULL;
  }
  
  // get the message id
  int id;
  if (socket.ReadINT32N(id) != sizeof(id))
  {
    return NULL;
  }
  if (id < MSG_ID_MIN || id >= MSG_ID_MAX)
  {
    return NULL;
  }
  
  // get the message data
  char* data       = new char[length];
  int   bytes_read = 0;
  while (bytes_read < length)
  {
    int cnt = socket.Read(data + bytes_read,length - bytes_read,bytes_read);
    if (cnt > 0)
    {
      bytes_read += cnt;
    }
    else
    {
      delete[]data;
      return NULL;
    }
  }
  
  // create the appropriate message instance
  switch (id)
  {
    case MSG_CLOSE_CONNECTION:
      return MsgCloseConnection::Create(data,length);
      break;
      
    case MSG_SET_SINGLE:
      return MsgSetSingle::Create(data,length);
      break;
      
    case MSG_SET_STRING:
      return MsgSetString::Create(data,length);
      break;
      
    case MSG_COLOR_SET:
      return MsgColorSet::Create(data,length);
      break;
      
    case MSG_COLOR_SEQ:
      return MsgColorSeq::Create(data,length);
      break;
    
    default:
      // should never be reached
      delete[]data;
      return NULL;
      break;
  }
  
  delete[]data;
}

void Message::Send(TCPSocket& socket) const
{
  // version
  int tmp = 1;
  socket.Send((const char*)&tmp,sizeof(tmp),0);
  
  // length
  tmp = GetMessageLength();
  socket.Send((const char*)&tmp,sizeof(tmp),0);
  
  // message id
  tmp = GetMessageID();
  socket.Send((const char*)&tmp,sizeof(tmp),0);
  
  // add in the subtype's data
  WriteContents(socket);
}
