#ifndef MESSAGE_H
#define MESSAGE_H

#include "../TCPSocket.h"
#include "MessageIDs.h"

/// <summary>
/// Base class for messages
/// </summary>
class Message
{
  public:
    Message();
    virtual ~Message();
    
    /// <summary>
    /// Reads a message from the socket
    /// </summary>
    /// <param name="socket">
    /// TCP socket to retreive a message from
    /// </param>
    /// <returns>
    /// pointer to the received messaged
    /// NULL on error
    /// </returns>
    static Message* Receive(TCPSocket& socket);
    
    /// <summary>
    /// Writes the message to the socket in binary form
    /// </summary>
    /// <param name="socket">
    /// TCP socket to send a message on
    /// </param>
    void Send(TCPSocket& socket) const;
    
    /// <summary>
    /// Retrieves the subtype's message id
    /// </summary>
    /// <returns>
    /// message id of the subtype
    /// </returns>
    virtual MessageIDs GetMessageID() const = 0;
    
  protected:
    /// <summary>
    /// Retrieves the number of bytes in the subtype's data when sending
    /// </summary>
    /// <returns>
    /// number of bytes in the message data section
    /// </returns>
    virtual int GetMessageLength() const = 0;
    
    /// <summary>
    /// Puts the subtype's data into the message via the specified socket
    /// </summary>
    /// <param name="socket">
    /// TCP socket to send a message on
    /// </param>
    virtual void WriteContents(TCPSocket& socket) const = 0;
    
  private:
    // disabled
    Message(const Message& cpy);
    Message& operator=(const Message& cpy);
};

#endif /* MESSAGE_H */
