#ifndef MSG_CLOSE_CONNECTION_H
#define MSG_CLOSE_CONNECTION_H

#include "Message.h"

/// <summary>
/// Message that informs a client that their connection is being closed
/// </summary>
class MsgCloseConnection : public Message
{
  public:
    MsgCloseConnection();
    ~MsgCloseConnection();
    
    /// <summary>
    /// Attempts to create an instance of this class based on the supplied data
    /// portion of a network message
    /// </summary>
    /// <returns>
    /// pointer to the message
    /// NULL on error
    /// </returns>
    static MsgCloseConnection* Create(char* data,int length);
    
    /// <summary>
    /// Retrieves the subtype's message id
    /// </summary>
    /// <returns>
    /// message id of the subtype
    /// </returns>
    MessageIDs GetMessageID() const;
    
  protected:
    /// <summary>
    /// Retrieves the number of bytes in the subtype's data when sending
    /// </summary>
    /// <returns>
    /// number of bytes in the message data section
    /// </returns>
    int GetMessageLength() const;
    
    /// <summary>
    /// Puts the any needed data into the message via the specified binary
    /// writer
    /// </summary>
    /// <param name="socket">
    /// TCP socket to send a message on
    /// </param>
    void WriteContents(TCPSocket& socket) const;
    
  private:
    // disabled
    MsgCloseConnection(const MsgCloseConnection& cpy);
    MsgCloseConnection& operator=(const MsgCloseConnection& cpy);
};

#endif /* MSG_CLOSE_CONNECTION_H */
