#ifndef MSG_COLOR_SEQ_H
#define MSG_COLOR_SEQ_H

#include <string>
#include <vector>
#include "Message.h"
#include "ParameterIDs.h"
#include "../ParticleSystem/Coloring/ColorSeqEntry.h"

/// <summary>
/// Passes an entire color set
/// </summary>
class MsgColorSeq : public Message
{
  public:
    MsgColorSeq(const std::vector<ColorSeqEntry>& val);
    ~MsgColorSeq();
    
    /// <summary>
    /// Attempts to create an instance of this class based on the supplied data
    /// portion of a network message
    /// </summary>
    /// <returns>
    /// pointer to the message
    /// NULL on error
    /// </returns>
    static MsgColorSeq* Create(char* data,int length);
    
    /// <summary>
    /// Retrieves the subtype's message id
    /// </summary>
    /// <returns>
    /// message id of the subtype
    /// </returns>
    MessageIDs GetMessageID() const;
    
    /// <summary>
    /// Retrieves the value
    /// </summary>
    /// <returns>
    /// value of the message
    /// </returns>
    std::vector<ColorSeqEntry>& GetValue();
    
  protected:
    /// <summary>
    /// Retrieves the number of bytes in the subtype's data when sending
    /// </summary>
    /// <returns>
    /// number of bytes in the message data section
    /// </returns>
    int GetMessageLength() const;
    
    /// <summary>
    /// Puts the any needed data into the message via the specified binary
    /// writer
    /// </summary>
    /// <param name="socket">
    /// TCP socket to send a message on
    /// </param>
    void WriteContents(TCPSocket& socket) const;
    
  private:
    // disabled
    MsgColorSeq();
    MsgColorSeq(const MsgColorSeq& cpy);
    MsgColorSeq& operator=(const MsgColorSeq& cpy);
    
    /// <summary>
    /// value
    /// </summary>
    std::vector<ColorSeqEntry> m_val;
};

#endif /* MSG_COLOR_SEQ_H */
