#ifndef MSG_COLOR_SET_H
#define MSG_COLOR_SET_H

#include <string>
#include <vector>
#include "Message.h"
#include "ParameterIDs.h"
#include "../ParticleSystem/Coloring/ColorSetEntry.h"

/// <summary>
/// Passes an entire color set
/// </summary>
class MsgColorSet : public Message
{
  public:
    MsgColorSet(const std::vector<ColorSetEntry>& val);
    ~MsgColorSet();
    
    /// <summary>
    /// Attempts to create an instance of this class based on the supplied data
    /// portion of a network message
    /// </summary>
    /// <returns>
    /// pointer to the message
    /// NULL on error
    /// </returns>
    static MsgColorSet* Create(char* data,int length);
    
    /// <summary>
    /// Retrieves the subtype's message id
    /// </summary>
    /// <returns>
    /// message id of the subtype
    /// </returns>
    MessageIDs GetMessageID() const;
    
    /// <summary>
    /// Retrieves the value
    /// </summary>
    /// <returns>
    /// value of the message
    /// </returns>
    std::vector<ColorSetEntry>& GetValue();
    
  protected:
    /// <summary>
    /// Retrieves the number of bytes in the subtype's data when sending
    /// </summary>
    /// <returns>
    /// number of bytes in the message data section
    /// </returns>
    int GetMessageLength() const;
    
    /// <summary>
    /// Puts the any needed data into the message via the specified binary
    /// writer
    /// </summary>
    /// <param name="socket">
    /// TCP socket to send a message on
    /// </param>
    void WriteContents(TCPSocket& socket) const;
    
  private:
    // disabled
    MsgColorSet();
    MsgColorSet(const MsgColorSet& cpy);
    MsgColorSet& operator=(const MsgColorSet& cpy);
    
    /// <summary>
    /// value
    /// </summary>
    std::vector<ColorSetEntry> m_val;
};

#endif /* MSG_COLOR_SET_H */
