﻿#ifndef MSG_CONNECTION_REJECTED_H
#define MSG_CONNECTION_REJECTED_H

#include "Message.h"

/// <summary>
/// Message that informs a client that their connection was rejected
/// </summary>
class MsgConnectionRejected : public Message
{
  public:
    MsgConnectionRejected();
    ~MsgConnectionRejected();
    
    /// <summary>
    /// Attempts to create an instance of this class based on the supplied data
    /// portion of a network message
    /// </summary>
    /// <returns>
    /// pointer to the message
    /// NULL on error
    /// </returns>
    static MsgConnectionRejected* Create(char* data,int length);
    
    /// <summary>
    /// Retrieves the message id
    /// </summary>
    /// <returns>
    /// message id
    /// </returns>
    MessageIDs GetMessageID() const;
    
  protected:
    /// <summary>
    /// Retrieves the number of bytes in the subtype's data when sending
    /// </summary>
    /// <returns>
    /// number of bytes in the message data section
    /// </returns>
    int GetMessageLength() const;
    
    /// <summary>
    /// Puts the any needed data into the message via the specified binary
    /// writer
    /// </summary>
    /// <param name="socket">
    /// TCP socket to send a message on
    /// </param>
    void WriteContents(TCPSocket& socket) const;
    
  private:
    // disabled
    MsgConnectionRejected(const MsgConnectionRejected& cpy);
    MsgConnectionRejected& operator=(const MsgConnectionRejected& cpy);
};

#endif /* MSG_CONNECTION_REJECTED_H */
