#ifndef MSG_SET_SINGLE_H
#define MSG_SET_SINGLE_H

#include "Message.h"
#include "ParameterIDs.h"

/// <summary>
/// Passes a single numeric value and the identifier for which value it is
/// </summary>
class MsgSetSingle : public Message
{
  public:
    MsgSetSingle(ParameterIDs id,float val);
    ~MsgSetSingle();
    
    /// <summary>
    /// Attempts to create an instance of this class based on the supplied data
    /// portion of a network message
    /// </summary>
    /// <returns>
    /// pointer to the message
    /// NULL on error
    /// </returns>
    static MsgSetSingle* Create(char* data,int length);
    
    /// <summary>
    /// Retrieves the subtype's message id
    /// </summary>
    /// <returns>
    /// message id of the subtype
    /// </returns>
    MessageIDs GetMessageID() const;
    
    /// <summary>
    /// Retrieves the parameter id
    /// </summary>
    /// <returns>
    /// parameter id of the message
    /// </returns>
    ParameterIDs GetParamID() const;
    
    /// <summary>
    /// Retrieves the value
    /// </summary>
    /// <returns>
    /// value of the message
    /// </returns>
    float GetValue() const;
    
  protected:
    /// <summary>
    /// Retrieves the number of bytes in the subtype's data when sending
    /// </summary>
    /// <returns>
    /// number of bytes in the message data section
    /// </returns>
    int GetMessageLength() const;
    
    /// <summary>
    /// Puts the any needed data into the message via the specified binary
    /// writer
    /// </summary>
    /// <param name="socket">
    /// TCP socket to send a message on
    /// </param>
    void WriteContents(TCPSocket& socket) const;
    
  private:
    // disabled
    MsgSetSingle();
    MsgSetSingle(const MsgSetSingle& cpy);
    MsgSetSingle& operator=(const MsgSetSingle& cpy);
    
    /// <summary>
    /// parameter id
    /// </summary>
    ParameterIDs m_id;
    
    /// <summary>
    /// value
    /// </summary>
    float m_val;
};

#endif /* MSG_SET_SINGLE_H */
