#include "ColoringSettings.h"
#include "../../GraphicsVector.h"
using namespace std;

XMFLOAT3 GetInitialColor(const ColoringSettings& color)
{
  XMFLOAT3 back(1,1,1);

  switch (color.active_mode)
  {
    case COLOR_SET:
      {
        float val   = (float)rand() / RAND_MAX;
        float total = 0;
        vector<ColorSetEntry>::const_iterator it = color.color_set.begin();
        while (it != color.color_set.end())
        {
          if (val <= it->GetChance() + total)
          {
            back = it->GetColor();
            break;
          }

          total += it->GetChance();
          
          ++it;
        }
      }
      break;

    case COLOR_SEQ:
      if (color.color_seq.size() > 0)
      {
        back = color.color_seq[0].GetColor();
      }
      break;
  }

  return back;
}

XMFLOAT3 GetColor(const ColoringSettings& color,float life)
{
  if (color.color_seq.size() == 0)
  {
    XMFLOAT3 back(1,1,1);
    return back;
  }
  
  float total = 0;
  vector<ColorSeqEntry>::const_iterator it = color.color_seq.begin();
  while (it != color.color_seq.end())
  {
    total += it->GetLife();

    if (total > life)
    {
      break;
    }
    
    ++it;
  }
  
  if (it == color.color_seq.end())
  {
    return color.color_seq.rbegin()->GetColor();
  }

  vector<ColorSeqEntry>::const_iterator next = it + 1;
  if (color.interpolate && color.color_seq.size() > 0
    && next != color.color_seq.end())
  {
    // determine how far between the current and next color sequence
    // entries the color_seq life is, in the range [0,1]
    float mul = (it->GetLife() + life - total)
      / it->GetLife();

    // use the multiplier to determine the color value
    return (1 - mul) * it->GetColor() + mul * next->GetColor();
  }
  else
  {
    return it->GetColor();
  }
}
