﻿#ifndef DIRECTIONAL_H
#define DIRECTIONAL_H

#include "DispersionBase.h"

/// <summary>
/// Sets particle movement to be a particular direction in emitter local
/// space
/// </summary>
class Directional : public DispersionBase
{
  public:
    /// <summary>
    /// Creates a directional dispersion instance
    /// </summary>
    Directional();
    
    /// <summary>
    /// Retrieves the raw vector so that its components can be set by the
    /// editing mechanism
    /// </summary>
    /// <returns>
    /// reference to the vector
    /// </returns>
    XMFLOAT4& GetRawVector();
    
    /// <summary>
    /// Determines the direction of a newly spawned particle
    /// </summary>
    /// <returns>
    /// unit vector representing the direction of the particle in emitter local
    /// space
    /// </returns>
    XMVECTOR GetDirection();
    
  private:
    /// <summary>
    /// direction to send particles in, in emitter local space
    /// </summary>
    XMFLOAT4 m_dir;
};

#endif /* DIRECTIONAL_H */
