﻿#ifndef DISPERSION_BASE_H
#define DISPERSION_BASE_H

#include <windows.h>
#include <xnamath.h>

/// <summary>
/// Base class for dispersion control classes
/// </summary>
/// <remarks>
/// A base class is used instead of delegates or lambda expressions to make
/// it easy for adding additional dispersion controls if plug-ins are
/// eventually supported
/// </remarks>
class DispersionBase
{
  public:
    virtual ~DispersionBase();
    
    /// <summary>
    /// Determines the direction of a newly spawned particle
    /// </summary>
    /// <returns>
    /// unit vector representing the direction of the particle in emitter local
    /// space
    /// </returns>
    virtual XMVECTOR GetDirection() = 0;
    
  private:
};

#endif /* DISPERSION_BASE_H */
