﻿#ifndef PARTIAL_SPHERE_H
#define PARTIAL_SPHERE_H

#include "DispersionBase.h"

/// <summary>
/// Sets particle movement to be in a partial sphere around the particle's
/// spawned location
/// </summary>
class PartialSphere : public DispersionBase
{
  public:
    PartialSphere();
    
    /// <summary>
    /// Determines the direction of a newly spawned particle
    /// </summary>
    /// <returns>
    /// unit vector representing the direction of the particle in emitter local
    /// space
    /// </returns>
    XMVECTOR GetDirection();
    
    /// <summary>
    /// Sets the min angle around the x-axis, in radians and in the emitter's
    /// local space, to use for the direction of particles
    /// </summary>
    /// <param name="val">
    /// value to set min x to
    /// </param>
    void SetMinX(float val);
    
    /// <summary>
    /// Sets the max angle around the x-axis, in radians and in the emitter's
    /// local space, to use for the direction of particles
    /// </summary>
    /// <param name="val">
    /// value to set max x to
    /// </param>
    void SetMaxX(float val);
    
    /// <summary>
    /// Sets the min angle around the y-axis, in radians and in the emitter's
    /// local space, to use for the direction of particles
    /// </summary>
    /// <param name="val">
    /// value to set min y to
    /// </param>
    void SetMinY(float val);
    
    /// <summary>
    /// Sets the max angle around the y-axis, in radians and in the emitter's
    /// local space, to use for the direction of particles
    /// </summary>
    /// <param name="val">
    /// value to set max y to
    /// </param>
    void SetMaxY(float val);
  
  private:
    /// <summary>
    /// min angle around the x-axis, in radians and in the emitter's local
    /// space, to use for the direction of particles
    /// </summary>
    float m_min_x;
    
    /// <summary>
    /// max angle around the x-axis, in radians and in the emitter's local
    /// space, to use for the direction of particles
    /// </summary>
    /// <remarks>
    /// Having this be a range would be more efficient when determing particle
    /// direction, but makes it more complicated for setting min/max for this
    /// rotation
    /// </remarks>
    float m_max_x;
    
    /// <summary>
    /// min angle around the y-axis, in radians and in the emitter's local
    /// space, to use for the direction of particles
    /// </summary>
    float m_min_y;
    
    /// <summary>
    /// maxn angle around the y-axis, in radians and in the emitter's local
    /// space, to use for the direction of particles
    /// </summary>
    /// <remarks>
    /// Having this be a range would be more efficient when determing particle
    /// direction, but makes it more complicated for setting min/max for this
    /// rotation
    /// </remarks>
    float m_max_y;
};

#endif /* PARTIAL_SPHERE_H */
