#ifndef EMITTER_SETTINGS_H
#define EMITTER_SETTINGS_H

#include <windows.h>
#include <xnamath.h>
#include "ParticleShapes.h"
#include "EmitterShapes/EmitterShapeBase.h"

/// <summary>
/// Stores settings for the particle emitter
/// </summary>
struct EmitterSettings
{
  /// <summary>
  /// World space position of the emitter
  /// </summary>
  XMFLOAT3 pos;
  
  /// <summary>
  /// Scale factors for the emitter, in emitter local space
  /// </summary>
  XMFLOAT3 scale;
  
  /// <summary>
  /// Emitter shape for determining the initial position of particles
  /// </summary>
  EmitterShapeBase* shape;
  
  /// <summary>
  /// Yaw for the rotation of the emitter's direction, in radians
  /// </summary>
  float yaw;
  
  /// <summary>
  /// Pitch for the rotation of the emitter's direction, in radians
  /// </summary>
  float pitch;
  
  /// <summary>
  /// Roll for the rotation of the emitter's direction, in radians
  /// </summary>
  float roll;
  
  /// <summary>
  /// Amount of time, in milliseconds, that the emitter should spawn particles
  /// for
  /// </summary>
  UINT spawn_time;

  /// <summary>
  /// How many particles per millisecond the emitter should generate
  /// </summary>
  float spawn_rate;
};

#endif /* EMITTER_SETTINGS_H */
