#ifndef PARTICLE_SETTINGS_H
#define PARTICLE_SETTINGS_H

#include <windows.h>
#include <xnamath.h>
#include "ParticleShapes.h"

/// <summary>
/// Stores settings for the particles
/// </summary>
struct ParticleSettings
{
  /// <summary>
  /// Shape of the particles
  /// </summary>
  ParticleShapes shape;
  
  /// <summary>
  /// How long in milliseconds particles should live for
  /// </summary>
  UINT lifetime;
  
  /// <summary>
  /// Size of the particles in units
  /// </summary>
  float size;
  
  /// <summary>
  /// Minimium speed in units/second that particles should move at
  /// </summary>
  float min_speed;
  
  /// <summary>
  /// Maximium speed in units/second that particles should move at
  /// </summary>
  float max_speed;
  
  /// <summary>
  /// Acceleration to apply to the speed of particles in units/second^2 in
  /// the emitter's local space
  /// </summary>
  XMFLOAT4 acceleration;
};

#endif /* PARTICLE_SETTINGS_H */
