#include <sstream>
#include "ServerSocket.h"
#include "log.h"
using namespace std;

WSADATA ServerSocket::s_winsock;

void ServerSocket::Init()
{
  int rc = WSAStartup(MAKEWORD(2,2),&s_winsock);
  if (rc)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: winsock failed to startup with error code " << rc
             << '\n';
      log_print(buffer.str().c_str());
    }
  }
}

void ServerSocket::Cleanup()
{
  WSACleanup();
}

ServerSocket* ServerSocket::Create(const char* port)
{
  int rc = 0;
  
  // prep the addrinfo structs for binding
  struct addrinfo* rslt = NULL;
  struct addrinfo hints;
  ZeroMemory(&hints,sizeof(hints));
  hints.ai_family   = AF_INET;
  hints.ai_socktype = SOCK_STREAM;
  hints.ai_protocol = IPPROTO_TCP;
  hints.ai_flags    = AI_PASSIVE;
  rc = getaddrinfo(NULL,port,&hints,&rslt);
  if (rc)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed to get addr info with error code " << rc
             << '\n';
      log_print(buffer.str().c_str());
    }
    return NULL;
  }
  
  // create server socket
  SOCKET sock;
  sock = socket(rslt->ai_family,rslt->ai_socktype,rslt->ai_protocol);
  if (sock == INVALID_SOCKET)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed to create socket with error code "
             << WSAGetLastError() << '\n';
      log_print(buffer.str().c_str());
    }
    freeaddrinfo(rslt);
    return NULL;
  }

  // bind the socket
  rc = bind(sock,rslt->ai_addr,rslt->ai_addrlen);
  if (rc == SOCKET_ERROR)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed to bind socket with error code "
             << WSAGetLastError() << '\n';
      log_print(buffer.str().c_str());
    }
    freeaddrinfo(rslt);
    closesocket(sock);
    return NULL;
  }
  
  // don't need the address info anymore
  freeaddrinfo(rslt);
  
  // make the socket ready to receive clients
  rc = listen(sock,SOMAXCONN);
  if (rc == SOCKET_ERROR)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed on listen with error code " << rc << '\n';
      log_print(buffer.str().c_str());
    }
    closesocket(sock);
    return NULL;
  }
  
  return new ServerSocket(sock);
}

ServerSocket::ServerSocket(SOCKET sock)
:m_sock(sock)
{
}

ServerSocket::~ServerSocket()
{
  closesocket(m_sock);
}

bool ServerSocket::PendingClient()
{
  fd_set set;
  FD_ZERO(&set);
  FD_SET(m_sock,&set);
  struct timeval timeout = { 0,0 };
  int rc = select(m_sock + 1,&set,NULL,NULL,&timeout);
  if (rc == SOCKET_ERROR)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed on select with error code " << WSAGetLastError()
             << '\n';
      log_print(buffer.str().c_str());
    }
    return false;
  }

  return rc == 1;
}

TCPSocket* ServerSocket::Accept()
{
  SOCKET sock = accept(m_sock,NULL,NULL);
  if (sock == INVALID_SOCKET)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed on accept with error code " << WSAGetLastError()
             << '\n';
      log_print(buffer.str().c_str());
    }
    return NULL;
  }

  return new TCPSocket(sock);
}
