#ifndef ServerSocket_H
#define ServerSocket_H

#include <winsock2.h>
#include <ws2tcpip.h>
#include "TCPSocket.h"

/// <summary>
/// TCP server socket
/// </summary>
class ServerSocket
{
  public:
    ~ServerSocket();
    
    /// <summary>
    /// Initializes winsock.  Must be called before creating any instances of
    /// this class
    /// </summary>
    static void Init();
    
    /// <summary>
    /// Cleans up winsock.  Must be called after all instances are destroyed,
    /// and new instances should not be created after this.
    /// </summary>
    static void Cleanup();
    
    /// <summary>
    /// Creates an instance bound to the specified port
    /// </summary>
    /// <param name="port">
    /// port to bind to in string form
    /// </param>
    /// <returns>
    /// pointer to the server socket on success
    /// NULL if an error occurred.
    /// </returns>
    static ServerSocket* Create(const char* port);
    
    /// <summary>
    /// A non-blocking check for if a client connection is waiting
    /// </summary>
    /// <returns>
    /// true if a client is waiting
    /// false otherwise
    /// </returns>
    bool PendingClient();
    
    /// <summary>
    /// Accepts a client connection, blocking if there is not one ready
    /// functions
    /// </summary>
    /// <returns>
    /// TCP client on success
    /// NULL if an error occurred
    /// </returns>
    TCPSocket* Accept();

  private:
    // disabled
    ServerSocket();
    ServerSocket(const ServerSocket& cpy);
    ServerSocket& operator=(const ServerSocket& cpy);
    
    /// <summary>
    /// Creates an instance attached to the specified server port
    /// </summary>
    /// <param name="sock">
    /// server socket
    /// </param>
    ServerSocket(SOCKET sock);

    /// <summary>
    /// var for winsock
    /// </summary>
    static WSADATA s_winsock;
    
    /// <summary>
    /// server socket
    /// </summary>
    SOCKET m_sock;
};

#endif /* ServerSocket_H */
