#include <sstream>
#include "TCPSocket.h"
#include "log.h"
using namespace std;

TCPSocket::TCPSocket(SOCKET sock)
:m_sock(sock)
{
}

TCPSocket::~TCPSocket()
{
  shutdown(m_sock,SD_SEND);
  closesocket(m_sock);
}

bool TCPSocket::DataPending()
{
  fd_set set;
  FD_ZERO(&set);
  FD_SET(m_sock,&set);
  struct timeval timeout = { 0,0 };
  int rc = select(m_sock + 1,&set,NULL,NULL,&timeout);
  if (rc == SOCKET_ERROR)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed on select with error code " << WSAGetLastError()
             << '\n';
      log_print(buffer.str().c_str());
    }
    return false;
  }

  return rc == 1;
}

int TCPSocket::Read(char* buffer,int count,int offset)
{
  int rc = recv(m_sock,buffer + offset,count,0);
  if (rc == SOCKET_ERROR)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed on read with error code " << WSAGetLastError()
             << '\n';
      log_print(buffer.str().c_str());
    }
    return -1;
  }
  
  return rc;
}

int TCPSocket::ReadINT32N(int32_t& val)
{
  int total = 0;
  int rc    = 0;
  while (total < sizeof(int32_t))
  {
    rc = Read((char*)&val,sizeof(int32_t) - total,total);
    if (rc > 0)
    {
      total += rc;
    }
    else
    {
      return rc;
    }
  }
  
  return total;
}

int TCPSocket::ReadINT32H(int32_t& val)
{
  int total = 0;
  int rc    = 0;
  while (total < sizeof(int32_t))
  {
    rc = Read((char*)&val,sizeof(int32_t) - total,total);
    if (rc > 0)
    {
      total += rc;
    }
    else
    {
      return rc;
    }
  }
  
  val = htonl(val);
  
  return total;
}

int TCPSocket::ReadUINT32N(uint32_t& val)
{
  int total = 0;
  int rc    = 0;
  while (total < sizeof(uint32_t))
  {
    rc = Read((char*)&val,sizeof(uint32_t) - total,total);
    if (rc > 0)
    {
      total += rc;
    }
    else
    {
      return rc;
    }
  }
  
  return total;
}

int TCPSocket::ReadUINT32H(uint32_t& val)
{
  int total = 0;
  int rc    = 0;
  while (total < sizeof(uint32_t))
  {
    rc = Read((char*)&val,sizeof(uint32_t) - total,total);
    if (rc > 0)
    {
      total += rc;
    }
    else
    {
      return rc;
    }
  }
  
  val = htonl(val);
  
  return total;
}

int TCPSocket::Send(const char* buffer,int count,int offset)
{
  int rc = send(m_sock,buffer + offset,count,0);
  if (rc == SOCKET_ERROR)
  {
    if (log_would_output())
    {
      ostringstream buffer;
      buffer << "Error: failed on send with error code " << WSAGetLastError()
             << '\n';
      log_print(buffer.str().c_str());
    }
    return -1;
  }
  
  return rc;
}
